@@ -26,6 +26,8 @@ import datetime
 import gobject
 import platform
 
+import dbus
+import dbus.service
 
 from elisa.core import __version__, version_info
 from elisa.core.config import Config, ConfigError
@@ -37,7 +39,7 @@ from elisa.core.components.message import Message
 
 from elisa.core import config_upgrader
 from elisa.core.plugin_registry import PluginRegistry
-from elisa.core import input_manager, media_manager, service_manager
+from elisa.core import input_manager, service_manager
 from elisa.core import metadata_manager, resource_manager
 from elisa.core import interface_controller
 from elisa.core.media_directory_helper import MediaDirectoryHelper
@@ -45,6 +47,9 @@ from elisa.core.media_directory_helper import MediaDirectoryHelper
 from twisted.internet import reactor, defer, task, threads
 from twisted.web import client
 
+# import pkg_resources after plugin_registry
+import pkg_resources
+
 # database
 try:
     from elisa.extern.storm_wrapper import store
@@ -52,10 +57,15 @@ try:
 except ImportError:
     store = None
 
+try:
+    from elisa.core.utils.update_checker import UpdateChecker
+except ImportError:
+    # could happen if the elisa http client plugin is missing
+    UpdateChecker = None
+
 
 from distutils.version import LooseVersion
 
-UPDATES_URL = "http://elisa.fluendo.com/updates/update.php?install_date=%(date)s&version=%(version)s&os=%(os)s&user_id=%(user_id)s"
 CONFIG_DIR = os.path.join(os.path.expanduser('~'), ".elisa-0.5")
 
 # We add to the configuration file name the version number in order to bypass
@@ -78,23 +88,31 @@ DEFAULT_CONFIG = """\
 [general]
 version = '%(version)s'
 install_date = '%(install_date)s'
-resource_providers = ['amazon.resource_provider:AmazonResourceProvider', 'discogs.discogs_resource:DiscogsResource', 'database.media_scanner:MediaScanner', 'weather.report_provider:WeatherReportProvider', 'base.local_resource:LocalResource', 'youtube.resource_provider:YoutubeResourceProvider', 'shoutcast.shoutcast_resource:ShoutcastResource', 'flickr.resource_provider:FlickrResourceProvider', 'wmd.wmd_resource:WMDResource', 'filtered_shares.filtered_shares_resource:FilteredSharesResource', 'coherence.coherence_resource:CoherenceResource', 'coherence.upnp_resource:UpnpResource', 'smbwin32.smbwin32_resource:SmbWin32Resource', 'elisa_updater.resource_provider:UpdaterResourceProvider', 'ipod.ipod_resource:IpodResource', 'hal.hal_resource:HALResource', 'avahi.avahi_resource_provider:AvahiResourceProvider', 'daap.daap_resource_provider:DaapResourceProvider', 'search.search_metaresource_provider:SearchMetaresourceProvider', 'yesfm.yesfm_resource:YesfmResource']
-metadata_providers = []
-service_providers = ['coherence.coherence_service:CoherenceService', 'osso.osso_service:OssoService', 'gnome.gnome_screensaver_service:GnomeScreensaverService','database.dbus_service:DatabaseDBusServiceProvider']
-input_providers = ['winremote.streamzap_input:StreamzapInput', 'lirc.lirc_input:LircInput']
 frontends = ['frontend1']
+disabled_plugins = []
 # database connection string. see https://storm.canonical.com/Manual
 database = '%(database_uri)s'
 
 [frontend1]
 frontend = 'pigment.pigment_frontend:PigmentFrontend'
-theme = 'elisa.plugins.poblesec'
+theme = 'elisa.plugins.hp'
 controller_path = '/poblesec'
 
 [directories]
 video = ['*default*']
 pictures = ['*default*']
 music = ['*default*']
+
+[poblesec.main:PoblesecController]
+viewport_buttons_mode = 'embedded'
+
+[poblesec.player_audio:AudioPlayerController]
+visualization = ''
+osd_timeout = '0'
+audio_sink = 'gconfaudiosink'
+
+[poblesec.player_video:VideoPlayerController]
+audio_sink = 'gconfaudiosink'
 """
 
 def setup_output_redirection(directory_path, log_filename=None):
@@ -110,6 +128,7 @@ class ComponentsLoadedMessage(Message):
     Sent when all components have been instantiated
     """
 
+
 class NewElisaVersionMessage(Message):
     """
     Sent when Application detects a new Elisa version is available
@@ -126,11 +145,17 @@ class NewElisaVersionMessage(Message):
         self.version = version
         self.installer_url = installer_url
 
+
+class FirstRunMessage(Message):
+    """
+    Sent when Elisa detects it is the first time it is being run.
+    """
+
+
 class BaseApplication(Loggable):
     config = None
     plugin_registry = None
     bus = None
-    media_manager = None
     metadata_manager = None
     service_manager = None
     input_manager = None
@@ -141,18 +166,24 @@ class BaseApplication(Loggable):
         # soon as possible
         super(BaseApplication, self).__init__()
 
+        self._first_run = False
         self._config_filename = self._get_config_filename(config)
+        if not os.path.exists(self._config_filename):
+            self._first_run = True
 
         # if running frozen on win32, redirect stdout/stderr to $CONFIG_DIR/elisa.log
         if hasattr(sys, "frozen"):
             setup_output_redirection(os.path.dirname(self._config_filename))
 
-        self.plugin_registry = self._create_plugin_registry(plugin_directories)
         self.config, self._install_date, self._user_id = self._load_config(config)
+        self.plugin_registry = self._create_plugin_registry(plugin_directories)
 
     def _create_plugin_registry(self, plugin_directories):
+        disabled_plugins = self.config.get_option('disabled_plugins',
+                                                  section='general',
+                                                  default=[])
         plugin_registry = PluginRegistry(plugin_directories)
-        plugin_registry.load_plugins()
+        plugin_registry.load_plugins(disabled_plugins)
         return plugin_registry
 
     def _get_config_filename(self, config):
@@ -203,18 +234,20 @@ class BaseApplication(Loggable):
         user_id = cfg.get_option('user_id', section='general', default='')
         return (cfg, install_date, user_id)
 
+
 class Application(BaseApplication):
     """ Application is the entry point of Elisa. It groups all the necessary
     elements needed for Elisa to run. It is in charge of instantiating a
     Config and a PluginRegistry. Application also provides access to
-    input events and data, and holds the user interfaces. It creates
-    various managers (InputManager, MediaManager...),
-    an InterfaceController and a DBBackend.
-
+    input events and data, and holds the user interfaces.
+ 
     @ivar plugin_registry:      loads and manages the plugins
     @type plugin_registry:      L{elisa.core.plugin_registry.PluginRegistry}
     @ivar config:               Application's configuration file, storing options
     @type config:               L{elisa.core.config.Config}
+    @ivar options:              Options passed on the command line when
+                                launching elisa
+    @type options:              L{elisa.core.options.Options}
     @ivar bus:                  DOCME
     @type bus:                  L{elisa.core.bus.Bus}
     @ivar metadata_manager:     DOCME
@@ -227,8 +260,6 @@ class Application(BaseApplication):
     @type interface_controller: L{elisa.core.interface_controller.InterfaceController}
     @ivar input_manager:        DOCME
     @type input_manager:        L{elisa.core.input_manager.InputManager}
-    @ivar media_manager:        DOCME
-    @type media_manager:        L{elisa.core.media_manager.MediaManager}
     @ivar store:                the access point to the database using storm
     @type store:                L{elisa.extern.twisted_storm.store.DeferredStore}
     """
@@ -236,7 +267,7 @@ class Application(BaseApplication):
     log_category = "application"
 
     def __init__(self, config_filename=None, show_tracebacks=False,
-                 splash=None):
+                 splash=None, options=None):
         """
         @param config_filename: the config filename to use. Can be absolute
                                 or relative path
@@ -259,44 +290,62 @@ class Application(BaseApplication):
         self._splash = splash
 
         self.store = None
+        self.update_checker = None
 
         self.bus = bus.Bus()
         self.service_manager = service_manager.ServiceManager()
         self.metadata_manager = metadata_manager.MetadataManager()
-        self.media_manager = media_manager.MediaManager(self.metadata_manager)
         self.resource_manager = resource_manager.ResourceManager()
         self.input_manager = input_manager.InputManager()
         self.interface_controller = interface_controller.InterfaceController()
+        self.options = options
 
-    def _check_updates(self):
-        distro = misc.get_os_name()
-        url = UPDATES_URL % {'date': self._install_date, 'version': __version__,
-                             'os': distro, 'user_id': self._user_id}
-        dfr = client.getPage(url)
+        if self._first_run:
+            # First time elisa is run with this configuration, we want to
+            # notify other components (so that the frontend can run a
+            # configuration wizard for instance).
+            self.bus.send_message(FirstRunMessage())
 
-        def got_result(result):
-            # epr_url = result
-            splitted = result.splitlines()
-            if len(splitted) == 1:
-                # hey, looks like new elisaweb hasn't been deployed yet.
-                user_id = ''
-                version = ''
-                installer_url = ''
-            else:
-                user_id, version, installer_url = splitted
+        import platform
+        system = platform.system()
+        if system == 'Windows':
+            from elisa.plugins.poblesec.windows_message import WindowsMessageSource
+            from win32con import WM_COPYDATA
+            self.windows_message_source = WindowsMessageSource()
+            winprocs = { WM_COPYDATA : self._on_copydata, }
+            self.windows_message_source.add_wndproc(winprocs)
 
-            if version and LooseVersion(version) > LooseVersion(__version__):
-                msg = NewElisaVersionMessage(version, installer_url)
-                self.bus.send_message(msg)
+    def _update_check_callback(self, results):
+        try:
+            version = results['version']
+        except KeyError:
+            version = None
 
-            # store user_id in config
-            self.config.set_option('user_id', user_id, section='general')
-            self._user_id = user_id
+        try:
+            user_id = results['user_id']
+        except KeyError:
+            user_id = None
 
-            # re-check for updates one day later
-            reactor.callLater(86400, threads.deferToThread, self._check_updates)
+        try:
+            installer_url = results['installer_url']
+        except KeyError:
+            installer_url = None
 
-        dfr.addCallback(got_result)
+        if version and LooseVersion(version) > LooseVersion(__version__):
+            msg = NewElisaVersionMessage(version, installer_url)
+            self.bus.send_message(msg)
+
+        # store user_id in config
+        self.config.set_option('user_id', user_id, section='general')
+        self._user_id = user_id
+
+        # store the country code
+        try:
+            self.config.set_option('country_code', results['country_code'],
+                    section='general')
+        except KeyError:
+            # country code not given in the response
+            pass
 
     def _load_exception_hook(self):
         """ Override the default system exception hook with our own
@@ -381,26 +430,33 @@ class Application(BaseApplication):
         if not os.path.exists(CONFIG_DIR):
             os.makedirs(CONFIG_DIR, 0755)
 
-        # we want to load the providers and then initialize the player and the
+        # we want to load the providers and then initialize the
         # interface controller
         self.media_directories = MediaDirectoryHelper()
 
         def loading_providers_done(result):
             return self.interface_controller.initialize()
 
-        managers = ( # manager, option in the config file
-                    (self.resource_manager, 'resource_providers'),
-                    (self.service_manager, 'service_providers'),
-                    (self.metadata_manager, 'metadata_providers'),
-                    (self.media_manager, 'media_providers'),
-                    (self.input_manager, 'input_providers'),
+        managers = ( # manager, entry_point
+                    (self.resource_manager, 'resource_provider'),
+                    (self.service_manager, 'service_provider'),
+                    (self.metadata_manager, 'metadata_provider'),
+                    (self.input_manager, 'input_provider'),
                    )
 
         def iter_managers(managers):
-            for manager, option in managers:
+            for manager, entry_point in managers:
+                entry_point = 'elisa.core.components.' + entry_point
                 self.debug("Loading providers for manager %s" % manager)
-                providers = self.config.get_option(option, section='general',
-                                                   default=[])
+
+                providers = []
+                for entry in pkg_resources.iter_entry_points(entry_point):
+                    # FIXME: consider only "enabled plugins"
+                    component_name = '%s:%s' % (entry.module_name,
+                            '.'.join(entry.attrs))
+
+                    providers.append(component_name)
+
                 if not providers:
                     self.debug("No providers to load for manager %s" % manager)
                     continue
@@ -413,6 +469,8 @@ class Application(BaseApplication):
                     loading_providers_done)
 
         def interface_initialized(result):
+            # Now that db/plugins are loaded, we can safely register dbus service name
+            self.dbus_name = dbus.service.BusName('com.fluendo.Elisa', dbus.SessionBus())
             self.bus.send_message(ComponentsLoadedMessage())
             self._close_splash_screen()
 
@@ -454,28 +512,32 @@ class Application(BaseApplication):
         self.input_manager.start()
         self.metadata_manager.start()
         self.resource_manager.start()
-        self.media_manager.start(seconds=5)
         self.service_manager.start()
 
-        # FIXME: use the http client for this
-        threads.deferToThread(self._check_updates)
+        if UpdateChecker:
+            self.update_checker = UpdateChecker(self._install_date,
+                    self._user_id,  __version__)
+            self.update_checker.start(self._update_check_callback)
 
     def stop(self, stop_reactor=True):
-        """Stop the application.
+        """
+        Stop the application.
 
-        @param stop_reactor:    stop the reactor after stopping the application
-        @type stop_reactor:     bool
-        @rtype:                 L{twisted.internet.defer.Deferred}
+        @param stop_reactor: stop the reactor after stopping the application
+        @type stop_reactor:  C{bool}
+        @rtype:              L{twisted.internet.defer.Deferred}
         """
         self._close_splash_screen()
 
+        if self.update_checker is not None:
+            self.update_checker.stop()
+
         def interface_controller_stopped(result):
             self.info("Stopping managers")
 
             manager_deferreds = []
             for manager in (self.service_manager, self.metadata_manager,
-                            self.media_manager, self.input_manager,
-                            self.resource_manager):
+                            self.input_manager, self.resource_manager):
                 manager_deferreds.append(defer.maybeDeferred(manager.stop))
             if self.store:
                 # FIXME: store is not closed properly: self.store.stop should be
@@ -510,7 +572,7 @@ class Application(BaseApplication):
                     reactor.stop()
 
         if self.running:
-            # stop the interface controller, then the player registry and the
+            # stop the interface controller and the
             # managers
             self.info("Stopping interface controller")
             dfr = self.interface_controller.stop()
@@ -522,5 +584,27 @@ class Application(BaseApplication):
     def _close_splash_screen(self):
         if self._splash != None:
             self._splash.destroy()
-
-
+            
+    def _on_copydata(self, hwnd, msg, wparam, lparam):
+        import ctypes
+        from ctypes import Structure, c_char, sizeof, addressof, memmove
+        from ctypes.wintypes import ULONG, DWORD
+        from mswin32.structures import COPYDATASTRUCT
+        
+        copydata = COPYDATASTRUCT()
+        memmove(addressof(copydata), lparam, sizeof(copydata))
+        
+        class COMMANDLINEDATA(Structure):
+            _fields_ = [
+                        ("data", c_char * copydata.cbData)
+                       ]
+        
+        commandlinedata = COMMANDLINEDATA()
+        memmove(addressof(commandlinedata), copydata.lpData, sizeof(commandlinedata))
+        frontend = self.interface_controller.frontends.values()[0]
+        main = frontend.retrieve_controllers('/poblesec')[0]
+        files = commandlinedata.data.split(" ")
+        if len(files) > 0:
+            main.play_file(files[0])
+        
+        return True
