@@ -32,8 +32,6 @@ import os
 import textwrap
 import tempfile
 
-from collections import defaultdict
-
 class ConfigError(Exception):
 
     def __init__(self, msg):
@@ -173,7 +171,7 @@ class Config(log.Loggable):
         @param section: the section name to search in
         @type section:  string
 
-        @raises L{SectionNotFound} when the given section is not existing
+        @raise SectionNotFound: when the given section does not exist
         """
         section_obj = self.get_section(section)
         if section_obj is None:
@@ -190,7 +188,7 @@ class Config(log.Loggable):
         @param section: the section name to search in
         @type section:  string
 
-        @raises L{SectionNotFound} when the given section is not existing
+        @raise SectionNotFound: when the given section does not exist
         """
         section_obj = self.get_section(section)
         if section_obj is None:
@@ -268,9 +266,11 @@ class Config(log.Loggable):
                               indict=section,
                               name=section_name)
 
-        real_section.comments = defaultdict(str, \
-                [(k, ['# %s' % line for line in textwrap.wrap(v, 77)]) \
-                                                for k,v in doc.iteritems()])
+        for key in section.iterkeys():
+            doc_value = doc.get(key, '')
+            real_section.comments[key] = \
+                    ['# %s' % line for line in textwrap.wrap(doc_value, 77)]
+
         self._config[section_name] = real_section
         return real_section
 
@@ -283,6 +283,17 @@ class Config(log.Loggable):
         if self._config.has_key(section_name):
             del self._config[section_name]
 
+    def delete_file(self):
+        """
+        Delete the configuration file on disk.
+        """
+        try:
+            os.remove(self._filename)
+        except OSError:
+            pass
+        else:
+            self.info("Deleted configuration file '%s'." % self._filename)
+
     def as_dict(self):
         """ Helper method to convert the Config instance to a dictionary
 
