@@ -54,6 +54,7 @@ class ConfigUpgrader(log.Loggable):
                          (('0.3.4', '0.3.4.1'), self._update_0_3_4_to_0_3_5),
                          (('0.5.7', '0.5.8'), self._update_0_5_7_to_0_5_8),
                          (('0.5.8', '0.5.9'), self._update_0_5_8_to_0_5_9),
+                         (('0.5.14', '0.5.15'), self._update_0_5_14_to_0_5_15),
                         ]
 
     def _get_version_index_upgrade(self, version_str):
@@ -133,6 +134,26 @@ class ConfigUpgrader(log.Loggable):
         self.debug("Doing a backup to the old config on %s", full_path)
         cfg.write(full_path)
 
+    def _update_0_5_14_to_0_5_15(self, cfg=None):
+        self.info('Upgrading from 0.5.14 to 0.5.15')
+        self._backup((0, 5, 14), cfg)
+        if not cfg:
+            cfg = self._current_config
+
+        # In 0.5.15, the *_providers keys have been removed from the
+        # configuration. Discovery of the providers is now done using the
+        # plugins' entry points.
+        provider_entries = ['resource_providers', 'metadata_providers',
+                            'service_providers', 'input_providers']
+        for providers_entry in provider_entries:
+            try:
+                cfg.del_option(providers_entry, section='general')
+            except config.SectionNotFound:
+                pass
+
+        self._current_version = '0.5.15'
+        return cfg
+
     def _update_0_5_8_to_0_5_9(self, cfg=None):
         self.info('Upgrading from 0.5.8 to 0.5.9')
         self._backup((0, 5, 8), cfg)
