@@ -77,7 +77,7 @@ class EggRegistry(Loggable):
         Elisa user directory.
 
         @param cache_file: the absolute path to the user's local cache file.
-        @type chache_file: a string
+        @type cache_file: a string
         @param static_sources: an alternative sources' list
         @type static_sources: list of strings
         @param plugin_dirs: the directory where to search for installed plugins
@@ -113,9 +113,7 @@ class EggRegistry(Loggable):
         if common.application:
             self.plugin_registry = common.application.plugin_registry
         else:
-            config_file = os.path.join(CONFIG_DIR, CONFIG_FILE)
-            self.plugin_registry = PluginRegistry(Config(config_file))
-            self.plugin_registry.load_plugins(self.plugin_dirs)
+            self.plugin_registry = PluginRegistry(self.plugin_dirs)
 
     def _load_static_repositories(self):
         # FIXME: get static repositories' sources from config file.
@@ -320,7 +318,7 @@ class EggRegistry(Loggable):
         @param plugin_name: the name of the plugin
         @type plugin_name: string
         @param plugin_version: the version of the plugin
-        @type plugin_name: string
+        @type plugin_version: string
         @param installed_plugins: a cached list of installed plugins
         @type installed_plugins: list of strings
         @return: the list of uninstalled dependecies
@@ -371,8 +369,6 @@ class EggRegistry(Loggable):
 
         Can filter by plugin name, returning at most one plugin.
 
-        @param plugin_name: the optional plugin name to search for
-        @type plugin_name: string
         @return: the list of installed plugins
         @rtype: list of L{epm.egg_plugin.EggPlugin}s
         """
