@@ -73,7 +73,7 @@ class InputManager(manager.Manager,gobject.GObject):
         self.info("Stopping")
         self._running = False
 
-        if self._poll_call is not None:
+        if self._poll_call.running:
             self._poll_call.stop()
         
         return manager.Manager.stop(self)
@@ -90,8 +90,6 @@ class InputManager(manager.Manager,gobject.GObject):
 
         @param event:         the event to process
         @type event:          L{elisa.core.input_event.InputEvent}
-        @param provider_path: the path of the InputProvider where the event is from
-        @type provider_path:  str
         """
         self.debug("Event received: %r", event)
 
