@@ -45,7 +45,7 @@ class InterfaceController(log.Loggable):
         """
         Read the configuration of the application and create the frontends
         that are specified in it. The configuration could for example look
-        like this:
+        like this::
 
             [general]
                 ...
@@ -64,11 +64,11 @@ class InterfaceController(log.Loggable):
         failure-Exception for that certain frontend is the
         L{FrontendOptionMissing}.
 
-        @rtype:     L{twisted.internet.defer.Deferred}
-        @return:    resulting in a list with a tuple of values where the first
-                    value is True/False to identify if it worked, the second
-                    one is the name of the frontend and the third one is either
-                    the frontend object if it worked or the Failure if it failed.
+        @rtype:  L{twisted.internet.defer.Deferred}
+        @return: resulting in a list with a tuple of values where the first
+                 value is True/False to identify if it worked, the second one
+                 is the name of the frontend and the third one is either the
+                 frontend object if it worked or the Failure if it failed.
         """
 
         self.info("Initializing")
