@@ -141,13 +141,13 @@ class MediaDirectoryHelper(object):
             dir.append(conf_uri)
             config.set_option(media_type, dir, section='directories')
 
-        if media_type in ('music', 'pictures'):
+        if media_type in ('music', 'pictures', 'video'):
             resource_manager = common.application.resource_manager
             # FIXME: hardcoded for the media scanner we have
             # FIXME: we can not cancel it after it was added
             scanner_uri = MediaUri("media_scanner://localhost/queue")
             try:
-                resource_manager.put(uri, scanner_uri)
+                resource_manager.put(uri, scanner_uri, section=media_type)
             except NoMatchingResourceProvider:
                 # media scanner not running
                 pass
