@@ -590,10 +590,9 @@ class MediaUri:
     def __cmp__(self, other_uri):
         """ Compare myself with another uri.
 
-        @see help of builtin cmp()
         @param other_uri: The URI I'm comparing myself with
         @type other_uri:  L{MediaUri}
-        @rtype:           int
+        @rtype:           C{int}
         @raise TypeError: When trying to compare with non-MediaUri object
         """
         if not isinstance(other_uri, MediaUri):
