@@ -17,6 +17,7 @@
 import inspect
 import os
 import sys
+import gst
 
 import pkg_resources
 
@@ -178,23 +179,25 @@ class PluginRegistry(Loggable):
             except ValueError:
                 pass
 
-    def load_plugins(self, plugin_names=None):
+    def _add_gstreamer_path(self, path):
+        gst.registry_get_default().scan_path(path)
+
+    def load_plugins(self, disabled_plugins=[]):
         """
         Load plugins from self.plugin_dirs.
 
-        This function should be called as early as possible at startup, _before_
-        using any plugin.
-        Note that this function runs without returning to the reactor for as
-        long as it takes. There's no point in making it return before it's
-        done as the plugin environment needs to be setup before any other part
-        of elisa can run correctly.
+        @attention: This function should be called as early as possible at
+                    startup, B{before} using any plugin.
 
-        You can optionally pass a list of plugin names to enable. If you don't
-        or pass None, all the available plugins are enabled.
+        @note: This function runs without returning to the reactor for as
+               long as it takes. There is no point in making it return before
+               it is done as the plugin environment needs to be set up before
+               any other part of elisa can run correctly.
 
-        @param plugin_names: the names of the plugins to enable. If None, all
-                             the available plugins are enabled.
-        @type plugin_names: sequence of strings or None
+        @note: By default, all the available plugins are enabled.
+
+        @param disabled_plugins: a list of plugins that should be disabled
+        @type disabled_plugins:  C{list} of C{str}
         """
         self.info('loading plugins from %s' % self.plugin_dirs)
 
@@ -224,7 +227,8 @@ class PluginRegistry(Loggable):
         # _deactivate_dist(). We use this map to keep track of the plugins that
         # are currently loaded and for which there's a new version available.
         reload_map = dict([(dist.key, [dist, False])
-                for dist in distributions + old_dists])
+                for dist in distributions + old_dists \
+                    if dist.key.startswith('elisa')])
 
         # add the plugins to the active working set
         for dist in distributions:
@@ -249,8 +253,12 @@ class PluginRegistry(Loggable):
             # add the distribution to the working set
             pkg_resources.working_set.add(dist)
 
-        if plugin_names:
-            plugin_names = set(plugin_names)
+            # distributions can ship gstreamer plugins in a directory called
+            # dist.location/gstreamer/
+            requirement = pkg_resources.Requirement.parse(dist.project_name)
+            if pkg_resources.resource_isdir(requirement, 'gstreamer'):
+                gst_path = pkg_resources.resource_filename(requirement, 'gstreamer')
+                self._add_gstreamer_path(gst_path)
 
         for key, (dist, to_reload) in reload_map.iteritems():
             if to_reload:
@@ -273,12 +281,9 @@ class PluginRegistry(Loggable):
                     rebuild(module)
                     self.info('reloaded module %s' % module)
  
-            if plugin_names is None or key in plugin_names:
-                self._plugin_status[key] = True
-            else:
-                self._plugin_status[key] = False 
+            self._plugin_status[key] = (key not in disabled_plugins)
 
-        self.info('loaded %d plugins' % len(distributions))
+        self.info('loaded %d plugins' % len(self._plugin_status))
 
     def enable_plugin(self, plugin_name):
         """
@@ -287,17 +292,41 @@ class PluginRegistry(Loggable):
         @param plugin_name: the name of the plugin to enable
         @type plugin_name: C{str}
         """
+        def update_configuration(result):
+            config = common.application.config
+            disabled_plugins = config.get_option('disabled_plugins',
+                                                 section='general',
+                                                 default=[])
+            try:
+                disabled_plugins.remove(plugin_name)
+            except ValueError:
+                pass
+            else:
+                config.set_option('disabled_plugins', disabled_plugins,
+                                  section='general')
+            return result
+
+        def send_message(result):
+            message = PluginStatusMessage(plugin_name,
+                    PluginStatusMessage.ActionType.ENABLED)
+            common.application.bus.send_message(message)
+
+            return defer.succeed(None)
+        
         try:
             if self._plugin_status[plugin_name]:
-                raise PluginAlreadyEnabled(plugin_name)
+                return defer.fail(PluginAlreadyEnabled(plugin_name))
         except KeyError:
-            raise PluginNotFound(plugin_name)
+            return defer.fail(PluginNotFound(plugin_name))
 
-        self._plugin_status[plugin_name] = True
-        message = PluginStatusMessage(plugin_name,
-                PluginStatusMessage.ActionType.ENABLED)
-        common.application.bus.send_message(message)
+        self._plugin_status[plugin_name] = True 
+        
+        dfr = self._call_hook(plugin_name, 'enable')
+        dfr.addCallback(update_configuration)
+        dfr.addCallback(send_message)
 
+        return dfr
+    
     def disable_plugin(self, plugin_name):
         """
         Disable a plugin.
@@ -305,27 +334,45 @@ class PluginRegistry(Loggable):
         @param plugin_name: the name of the plugin to disable
         @type plugin_name: C{str}
         """
+        def update_configuration(result):
+            config = common.application.config
+            disabled_plugins = config.get_option('disabled_plugins',
+                                                 section='general',
+                                                 default=[])
+            if plugin_name not in disabled_plugins:
+                disabled_plugins.append(plugin_name)
+                config.set_option('disabled_plugins', disabled_plugins,
+                                  section='general')
+            return result
+
+        def send_message(result):
+            message = PluginStatusMessage(plugin_name,
+                    PluginStatusMessage.ActionType.DISABLED)
+            common.application.bus.send_message(message)
 
         try:
             if not self._plugin_status[plugin_name]:
-                raise PluginAlreadyDisabled(plugin_name)
+                return defer.fail(PluginAlreadyDisabled(plugin_name))
         except KeyError:
-            raise PluginNotFound(plugin_name)
+            return defer.fail(PluginNotFound(plugin_name))
  
         self._plugin_status[plugin_name] = False
-        message = PluginStatusMessage(plugin_name,
-                PluginStatusMessage.ActionType.DISABLED)
-        common.application.bus.send_message(message)
-    
+        
+        dfr = self._call_hook(plugin_name, 'disable')
+        dfr.addCallback(update_configuration)
+        dfr.addCallback(send_message)
+
+        return dfr
+
     def get_plugins(self):
         """
         Get the list of available plugins.
 
-        This call returns (plugin_name, status) tuples, where status is True if
-        the plugin is enabled, False otherwise.
+        This call returns (plugin_name, status) tuples, where status is C{True}
+        if the plugin is enabled, C{False} otherwise.
 
-        @return generator yielding (plugin_name, status) tuples
-        @rtype: C{generator}
+        @return: a generator yielding (plugin_name, status) tuples
+        @rtype:  C{generator}
         """
         return self._plugin_status.iteritems()
 
@@ -389,3 +436,17 @@ class PluginRegistry(Loggable):
         self.debug('got class %s, calling create()'
                 % reflect.qual(component_class))
         return component_class.create(config, **kwargs)
+
+    def _call_hook(self, plugin_name, hook_name):
+        req = pkg_resources.Requirement.parse(plugin_name)
+        dist = pkg_resources.working_set.find(req)
+
+        try:
+            hook = dist.load_entry_point('elisa.core.plugin_registry',
+                    hook_name)
+        except ImportError:
+            dfr = defer.succeed(None)
+        else:
+            dfr = hook()
+
+        return dfr
