@@ -97,8 +97,9 @@ class DeferFunctionTesting(unittest.TestCase):
     def testCompability(self):
         # test that all given classes and methods of twisted defer are in our
         # defer
+        msg = "We miss %s" 
         for item in twisted_defer.__all__:
-            self.assertTrue(hasattr(defer, item))
+            self.assertTrue(hasattr(defer, item), msg % item)
 
 
     def testAccumulator(self):
