@@ -31,6 +31,7 @@ import pkg_resources
 class FakeApplication(object):
     def __init__(self, config_file, plugin_registry, bus):
         self.config = Config(config_file)
+        self.config.set_section('general')
         self.plugin_registry = plugin_registry
         self.bus = bus
 
@@ -41,11 +42,19 @@ class FakeBus(object):
     def send_message(self, message):
         self.messages.append(message)
 
+class PluginRegistryMock(PluginRegistry):
+    def __init__(self, *args, **kw):
+        super(PluginRegistryMock, self).__init__(*args, **kw)
+        self.gst_paths = []
+
+    def _add_gstreamer_path(self, path):
+        self.gst_paths.append(path)
+
 class TestPluginRegistryMixin(object):
     plugin_registry = None
     old_application = None
 
-    def setUpClass(self): 
+    def setUp(self): 
         # do everything under _trial_temp/test_plugin_registry
         self.test_dir = os.path.abspath('test_plugin_registry')
         pkg_resources.ensure_directory(self.test_dir)
@@ -53,19 +62,16 @@ class TestPluginRegistryMixin(object):
 
         # copy the test plugins
         self.copy_plugins()
-        
+
         self.install_plugin('test_simple-0.1', '0.1/test_simple')
 
-    def tearDownClass(self):
-        self.uninstall_plugin('test_simple-0.1')
-
-    def setUp(self):
         # setup common.application
         self.patch_application()
-    
+
     def tearDown(self):
+        self.uninstall_plugin('test_simple-0.1')
         self.unpatch_application()
-    
+
     def patch_application(self):
         """
         Setup common.application, saving the old application object.
@@ -110,11 +116,11 @@ class TestPluginRegistryMixin(object):
 class TestPluginRegistry(TestPluginRegistryMixin, TestCase):
     plugin_registry = None
 
-    def setUpClass(self):
-        TestPluginRegistryMixin.setUpClass(self)
+    def setUp(self):
+        TestPluginRegistryMixin.setUp(self)
          
         # create the PluginRegistry instance used by the tests
-        self.plugin_registry = PluginRegistry([self.plugins_dir])
+        self.plugin_registry = PluginRegistryMock([self.plugins_dir])
         self.plugin_registry.load_plugins()
 
     def test_import(self):
@@ -124,6 +130,15 @@ class TestPluginRegistry(TestPluginRegistryMixin, TestCase):
         Check that the normal import statement works correctly.
         """
         from elisa.plugins.test_simple.test_module import TestComponent
+    
+    def test_gstreamer_plugin(self):
+        """
+        Import a component from a plugin.
+
+        Check that the normal import statement works correctly.
+        """
+        from elisa.plugins.test_simple.test_module import TestComponent
+        self.failUnless(self.plugin_registry.gst_paths)
 
     def test_create(self):
         """
@@ -249,100 +264,165 @@ class TestPluginRegistry(TestPluginRegistryMixin, TestCase):
 
         return d
 
+    test_hot_upgrade.todo = "Reimplement Hot Upgrade"
+
 class TestPluginActivation(TestPluginRegistryMixin, TestCase):
     def setUp(self):
         TestPluginRegistryMixin.setUp(self)
-         
-        # create the PluginRegistry instance used by the tests
+
+        # Create the PluginRegistry instance used by the tests
         self.plugin_registry = PluginRegistry([self.plugins_dir])
-        
-        # don't enable any plugins
-        self.plugin_registry.load_plugins([])
+
+        # Disable all the plugins
+        plugins = [plugin.key for plugin in pkg_resources.working_set \
+                   if plugin.key.startswith('elisa')]
+        plugins.append('elisa-plugin-test-simple')
+        self.plugin_registry.load_plugins(plugins)
+        config = common.application.config
+        config.set_option('disabled_plugins', ['elisa-plugin-test-simple'],
+                          section='general')
+
+    def check_disabled_plugins(self, expected):
+        config = common.application.config
+        disabled_plugins = config.get_option('disabled_plugins',
+                                             section='general', default=[])
+        self.failUnlessEqual(disabled_plugins, expected)
 
     def test_enable_plugin(self):
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 0)
+        def enable_plugin(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 0)
+            self.check_disabled_plugins(['elisa-plugin-test-simple'])
+            return self.plugin_registry.enable_plugin('elisa-plugin-test-simple')
 
-        self.plugin_registry.enable_plugin('elisa-plugin-test-simple')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 1)
-        # we should get a plugin enabled message
-        self.failUnlessEqual(len(self.bus.messages), 1)
-        message = self.bus.messages[0]
-        self.failUnlessEqual(message.plugin_name, 'elisa-plugin-test-simple')
-        self.failUnlessEqual(message.action, message.ActionType.ENABLED)
-        
-        self.failUnlessRaises(PluginAlreadyEnabled,
-                self.plugin_registry.enable_plugin,
-                'elisa-plugin-test-simple')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 1)
-        # we shouldn't get a message here
-        self.failUnlessEqual(len(self.bus.messages), 1)
+        def check_plugins(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 1)
+            # we should get a plugin enabled message
+            self.failUnlessEqual(len(self.bus.messages), 1)
+            message = self.bus.messages[0]
+            self.failUnlessEqual(message.plugin_name, 'elisa-plugin-test-simple')
+            self.failUnlessEqual(message.action, message.ActionType.ENABLED)
+            self.check_disabled_plugins([])
+
+        def enable_again(result):
+            enable_dfr = self.plugin_registry.enable_plugin('elisa-plugin-test-simple')
+            dfr = self.failUnlessFailure(enable_dfr, PluginAlreadyEnabled)
+            return dfr
+
+        def check_again_plugins(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 1)
+            # we shouldn't get a message here
+            self.failUnlessEqual(len(self.bus.messages), 1)
+            self.check_disabled_plugins([])
+
+        dfr = defer.succeed(None)
+        dfr.addCallback(enable_plugin)
+        dfr.addCallback(check_plugins)
+        dfr.addCallback(enable_again)
+        dfr.addCallback(check_again_plugins)
+        return dfr
 
     def test_enable_not_found(self):
+        def check_plugins(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 0)
+            # we shouldn't get a message here
+            self.failUnlessEqual(len(self.bus.messages), 0)        
+            self.check_disabled_plugins(['elisa-plugin-test-simple'])
+
         plugins = list(self.plugin_registry.get_enabled_plugins())
         self.failUnlessEqual(len(plugins), 0)
- 
-        self.failUnlessRaises(PluginNotFound,
-                self.plugin_registry.enable_plugin,
-                'a-plugin-named-like-this-cant-possibly-exist')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 0)
-        # we shouldn't get a message here
-        self.failUnlessEqual(len(self.bus.messages), 0)        
+        self.check_disabled_plugins(['elisa-plugin-test-simple'])
+
+        enable_dfr = self.plugin_registry.enable_plugin('icannotpossiblyexist')
+        dfr = self.failUnlessFailure(enable_dfr, PluginNotFound)
+        dfr.addCallbacks(check_plugins)
+        return dfr
 
     def test_disable_plugin(self):
+        def disable_not_enabled(result):
+            # start empty
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 0)
+            self.check_disabled_plugins(['elisa-plugin-test-simple'])
+
+            # disable a plugin that is not enabled
+            disable_dfr = self.plugin_registry.disable_plugin('elisa-plugin-test-simple')
+            dfr = self.failUnlessFailure(disable_dfr, PluginAlreadyDisabled)
+            return dfr
+
+        def check_no_plugins_no_messages(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 0)
+            # we shouldn't get a message here
+            self.failUnlessEqual(len(self.bus.messages), 0)
+            self.check_disabled_plugins(['elisa-plugin-test-simple'])
+
+        def enable_plugin(result):
+            # enable a plugin
+            return self.plugin_registry.enable_plugin('elisa-plugin-test-simple')
+
+        def check_one_enabled(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 1)
+            # we should get a plugin enabled message
+            self.failUnlessEqual(len(self.bus.messages), 1)
+            message = self.bus.messages[0]
+            self.failUnlessEqual(message.plugin_name, 'elisa-plugin-test-simple')
+            self.failUnlessEqual(message.action, message.ActionType.ENABLED)
+            self.check_disabled_plugins([])
+
+        def disable_enabled(result):
+            # disable the plugin
+            return self.plugin_registry.disable_plugin('elisa-plugin-test-simple')
+
+        def check_no_plugins_one_disabled_message(result):
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 0)
+            message = self.bus.messages[1]
+            self.failUnlessEqual(message.plugin_name, 'elisa-plugin-test-simple')
+            self.failUnlessEqual(message.action, message.ActionType.DISABLED)
+            self.check_disabled_plugins(['elisa-plugin-test-simple'])
+
+        dfr = defer.succeed(None)
+        dfr.addCallback(disable_not_enabled)
+        dfr.addCallback(check_no_plugins_no_messages)
+        dfr.addCallback(enable_plugin)
+        dfr.addCallback(check_one_enabled)
+        dfr.addCallback(disable_enabled)
+        dfr.addCallback(check_no_plugins_one_disabled_message)
+        return dfr
+
+    def test_disable_not_found(self):
+        def check_plugins(result):
+            # check that it's still empty
+            plugins = list(self.plugin_registry.get_enabled_plugins())
+            self.failUnlessEqual(len(plugins), 0)
+            # we shouldn't get a message here
+            self.failUnlessEqual(len(self.bus.messages), 0)
+            self.check_disabled_plugins(['elisa-plugin-test-simple'])
+
         # start empty
         plugins = list(self.plugin_registry.get_enabled_plugins())
         self.failUnlessEqual(len(plugins), 0)
-        
-        # disable a plugin that is not enabled
-        self.failUnlessRaises(PluginAlreadyDisabled,
-                self.plugin_registry.disable_plugin,
-                'elisa-plugin-test-simple')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 0)
-        # we shouldn't get a message here
-        self.failUnlessEqual(len(self.bus.messages), 0)
 
-        # enable a plugin
-        self.plugin_registry.enable_plugin('elisa-plugin-test-simple')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 1)
-        # we should get a plugin enabled message
-        self.failUnlessEqual(len(self.bus.messages), 1)
-        message = self.bus.messages[0]
-        self.failUnlessEqual(message.plugin_name, 'elisa-plugin-test-simple')
-        self.failUnlessEqual(message.action, message.ActionType.ENABLED)
-
-        # disable the plugin
-        self.plugin_registry.disable_plugin('elisa-plugin-test-simple')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 0)
-        message = self.bus.messages[1]
-        self.failUnlessEqual(message.plugin_name, 'elisa-plugin-test-simple')
-        self.failUnlessEqual(message.action, message.ActionType.DISABLED)
-
-    def test_disable_not_found(self):
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 0)
- 
-        self.failUnlessRaises(PluginNotFound,
-                self.plugin_registry.disable_plugin,
-                'a-plugin-named-like-this-cant-possibly-exist')
-        plugins = list(self.plugin_registry.get_enabled_plugins())
-        self.failUnlessEqual(len(plugins), 0)
-        # we shouldn't get a message here
-        self.failUnlessEqual(len(self.bus.messages), 0)
+        # disable something that is not enabled
+        disable_dfr = self.plugin_registry.disable_plugin('icannotpossiblyexist')
+        dfr = self.failUnlessFailure(disable_dfr, PluginNotFound)
+        dfr.addCallback(check_plugins)
+        return dfr
 
 class TestPluginLoadActivation(TestPluginRegistryMixin, TestCase):
     def test_load_plugins(self):
         self.plugin_registry = PluginRegistry([self.plugins_dir])
-        
-        # enable one plugin
-        self.plugin_registry.load_plugins(['elisa-plugin-test-simple'])
+
+        # Disable all plugins but our test one
+        plugins = [plugin.key for plugin in pkg_resources.working_set \
+                   if plugin.key.startswith('elisa') and \
+                   plugin.key != 'elisa-plugin-test-simple']
+        self.plugin_registry.load_plugins(plugins)
 
         plugins = list(self.plugin_registry.get_enabled_plugins())
         self.failUnlessEqual(len(plugins), 1)
-
