@@ -15,7 +15,7 @@
 
 from elisa.core.tests.elisa_test_case import ElisaTestCase
 from elisa.core.service_manager import ServiceManager
-from elisa.base_components.service_provider import ServiceProvider
+from elisa.core.components.service_provider import ServiceProvider
 from elisa.core.manager import CannotUnregister
 
 
@@ -31,17 +31,12 @@ class SimpleService(ServiceProvider):
 
 class TestServiceManager(ElisaTestCase):
 
-
     def setUp(self):
         ElisaTestCase.setUp(self)
         self._manager = ServiceManager()
         self._provider = SimpleService()
         self._manager.register_component(self._provider)
 
-    def test_component_registered(self):
-        reg = self._manager.component_registered(self._provider)
-        self.assertEquals(reg, True)
-
     def test_component_start(self):
         self._provider.running = False
         self._manager.start()
@@ -51,13 +46,3 @@ class TestServiceManager(ElisaTestCase):
         self._provider.running = True
         self._manager.stop()
         self.assertEquals(self._provider.running, False)
-
-    def test_component_unregister(self):
-        service = SimpleService()
-        self._manager.register_component(service)
-        self._manager.unregister_component(service)
-        result = self._manager.component_registered(service)
-        self.assertEquals(result, False)
-
-        self.failUnlessRaises(CannotUnregister, self._manager.unregister_component, service)
-
