@@ -23,7 +23,7 @@ import fnmatch
 import time
 import types
 import traceback
-    
+
 # environment variables controlling levels for each category
 _DEBUG = "*:1"
 # name of the environment variable controlling our logging
@@ -38,19 +38,19 @@ _categories = {}
 _log_handlers = []
 _log_handlers_limited = []
 
-_initialized = []
+_initialized = False
 
 _stdout = None
 _stderr = None
 _old_hup_handler = None
-   
+
 
 # public log levels
-ERROR = 1
-WARN = 2
-INFO = 3
-DEBUG = 4
-LOG = 5
+(ERROR,
+ WARN,
+ INFO,
+ DEBUG,
+ LOG) = range(1, 6)
 
 COLORS = {ERROR: 'RED',
           WARN: 'YELLOW',
@@ -90,12 +90,12 @@ def registerCategory(category):
     for chunk in chunks:
         if not chunk:
             continue
-        if ':' in chunk: 
+        if ':' in chunk:
             spec, value = chunk.split(':')
         else:
             spec = '*'
             value = chunk
-            
+
         # our glob is unix filename style globbing, so cheat with fnmatch
         # fnmatch.fnmatch didn't work for this, so don't use it
         if category in fnmatch.filter((category, ), spec):
@@ -121,6 +121,36 @@ def getCategoryLevel(category):
         registerCategory(category)
     return _categories[category]
 
+def setLogSettings(state):
+    """Update the current log settings.
+    This can restore an old saved log settings object returned by
+    getLogSettings
+    @param state: the settings to set
+    """
+
+    global _DEBUG
+    global _log_handlers
+    global _log_handlers_limited
+
+    (_DEBUG,
+     _categories,
+     _log_handlers,
+     _log_handlers_limited) = state
+
+    for category in _categories:
+        registerCategory(category)
+
+def getLogSettings():
+    """Fetches the current log settings.
+    The returned object can be sent to setLogSettings to restore the
+    returned settings
+    @returns: the current settings
+    """
+    return (_DEBUG,
+            _categories,
+            _log_handlers,
+            _log_handlers_limited)
+
 def _canShortcutLogging(category, level):
     if _log_handlers:
         # we have some loggers operating without filters, have to do
@@ -144,7 +174,7 @@ def scrubFilename(filename):
 def getFileLine(where=-1):
     """
     Return the filename and line number for the given location.
-    
+
     If where is a negative integer, look for the code entry in the current
     stack that is the given number of frames above this module.
     If where is a function, look for the code entry of the function.
@@ -157,7 +187,7 @@ def getFileLine(where=-1):
     """
     co = None
     lineno = None
-    
+
     if isinstance(where, types.FunctionType):
         co = where.func_code
         lineno = co.co_firstlineno
@@ -193,7 +223,7 @@ def ellipsize(o):
 
     r = r[:60] + ' ... ' + r[-15:]
     return r
- 
+
 def getFormatArgs(startFormat, startArgs, endFormat, endArgs, args, kwargs):
     """
     Helper function to create a format and args to use for logging.
@@ -208,7 +238,8 @@ def getFormatArgs(startFormat, startArgs, endFormat, endArgs, args, kwargs):
     debugArgs.extend(endArgs)
     format = startFormat \
               + ', '.join(('%s', ) * len(args)) \
-              + ', '.join(('%s=%r, ', ) * len(kwargs)) \
+              + (kwargs and ', ' or '') \
+              + ', '.join(('%s=%r', ) * len(kwargs)) \
               + endFormat
     return format, debugArgs
 
@@ -266,7 +297,7 @@ def doLog(level, object, category, format, args, where=-1,
             raise SystemError, "handler %r raised a TypeError" % handler
 
         return ret
-    
+
 def errorObject(object, cat, format, *args):
     """
     Log a fatal error message in the given category.
@@ -383,7 +414,7 @@ def init(envVarName, enableColorOutput=False):
     """
     global _initialized
 
-    if envVarName in _initialized:
+    if _initialized:
         return
 
     global _ENV_VAR_NAME
@@ -399,14 +430,14 @@ def init(envVarName, enableColorOutput=False):
         setDebug(os.environ[envVarName])
     addLimitedLogHandler(stderrHandler)
 
-    _initialized.append(envVarName)
+    _initialized = True
 
 def setDebug(string):
     """Set the DEBUG string.  This controls the log output."""
     global _DEBUG
     global _ENV_VAR_NAME
     global _categories
-    
+
     _DEBUG = string
     debug('log', "%s set to %s" % (_ENV_VAR_NAME, _DEBUG))
 
@@ -438,10 +469,10 @@ def reset():
     Resets the logging system, removing all log handlers.
     """
     global _log_handlers, _log_handlers_limited, _initialized
-    
+
     _log_handlers = []
     _log_handlers_limited = []
-    _initialized = []
+    _initialized = False
 
 def addLogHandler(func):
     """
@@ -461,7 +492,7 @@ def addLogHandler(func):
 
     if func not in _log_handlers:
         _log_handlers.append(func)
-        
+
 def addLimitedLogHandler(func):
     """
     Add a custom log handler.
@@ -472,14 +503,14 @@ def addLimitedLogHandler(func):
                  getLevelName(level) to get a printable name for the log level.
     @type func:  a callable function
 
-    @raises TypeError: TypeError if func is not a callable    
+    @raises TypeError: TypeError if func is not a callable
     """
     if not callable(func):
         raise TypeError, "func must be callable"
 
     if func not in _log_handlers_limited:
         _log_handlers_limited.append(func)
-    
+
 def removeLogHandler(func):
     """
     Remove a registered log handler.
@@ -494,7 +525,7 @@ def removeLogHandler(func):
     """
     _log_handlers.remove(func)
 
-    
+
 def removeLimitedLogHandler(func):
     """
     Remove a registered limited log handler.
@@ -551,23 +582,27 @@ def reopenOutputFiles():
     Reopens the stdout and stderr output files, as set by
     L{outputToFiles}.
     """
-    if not (_stdout and _stderr):
+    if not _stdout and not _stderr:
         debug('log', 'told to reopen log files, but log files not set')
         return
 
-    oldmask = os.umask(0026)
-    try:
-        so = open(_stdout, 'a+') 
-        se = open(_stderr, 'a+', 0)
-    finally:
-        os.umask(oldmask)
+    def reopen(name, fileno, *args):
+        oldmask = os.umask(0026)
+        try:
+            f = open(name, 'a+', *args)
+        finally:
+            os.umask(oldmask)
 
-    os.dup2(so.fileno(), sys.stdout.fileno()) 
-    os.dup2(se.fileno(), sys.stderr.fileno())
+        os.dup2(f.fileno(), fileno)
 
-    debug('log', 'opened log %r', _stderr)
+    if _stdout:
+        reopen(_stdout, sys.stdout.fileno())
 
-def outputToFiles(stdout, stderr):
+    if _stderr:
+        reopen(_stderr, sys.stderr.fileno(), 0)
+        debug('log', 'opened log %r', _stderr)
+
+def outputToFiles(stdout=None, stderr=None):
     """
     Redirect stdout and stderr to named files.
 
@@ -611,21 +646,21 @@ class Loggable:
     """
 
     logCategory = 'default'
-    
+
     def error(self, *args):
         """Log an error.  By default this will also raise an exception."""
         if _canShortcutLogging(self.logCategory, ERROR):
             return
         errorObject(self.logObjectName(), self.logCategory,
             *self.logFunction(*args))
-        
+
     def warning(self, *args):
         """Log a warning.  Used for non-fatal problems."""
         if _canShortcutLogging(self.logCategory, WARN):
             return
         warningObject(self.logObjectName(), self.logCategory,
             *self.logFunction(*args))
-        
+
     def info(self, *args):
         """Log an informational message.  Used for normal operation."""
         if _canShortcutLogging(self.logCategory, INFO):
@@ -800,7 +835,7 @@ class TwistedLogObserver(Loggable):
                     if r:
                         self.debug("Failure of type %r, ignoring" % failureType)
                         return
-                    
+
                 self.log("Failure %r" % f)
 
                 method = debug # tracebacks from errors at debug level
